﻿//

#include "stdafx.h"
#include "BlendDlg.h"
#include "afxdialogex.h"

IMPLEMENT_DYNAMIC(BlendDlg, CDialogEx)

BlendDlg::BlendDlg(CHPSView* in_view, void* pProcess, CWnd* pParent /*=nullptr*/)
	: CDialogEx(IDD_BLEND_DIALOG, pParent)
	,view(in_view)
	, m_dBlendR(10)
{
	Create(IDD_BLEND_DIALOG, pParent);

#ifdef USING_EXCHANGE_PARASOLID
	m_pProcess = (ExPsProcess*)pProcess;
	HPS::Component::ComponentType targetComp = HPS::Component::ComponentType::ParasolidTopoEdge;
#else
	m_pProcess = (ExProcess*)pProcess;
	HPS::Component::ComponentType targetComp = HPS::Component::ComponentType::ExchangeTopoEdge;
#endif

	m_pCmdOp = new ClickEntitiesCmdOp(targetComp, view, m_pProcess, false, HPS::MouseButtons::ButtonLeft());

	view->GetCanvas().GetFrontView().GetOperatorControl().Push(m_pCmdOp);

}

BlendDlg::~BlendDlg()
{

	m_pCmdOp->DetachView();
}

BOOL BlendDlg::OnInitDialog()
{
	BOOL ret = CDialog::OnInitDialog();

	UINT timerID = 1;
	UINT interval = 1 * 200;
	m_timerID = SetTimer(timerID, interval, NULL);

	return ret;
}

void BlendDlg::OnOK()
{
	auto t0 = std::chrono::system_clock::now();
	
	m_pCmdOp->Unhighlight();

	HPS::ComponentArray selCompArr = m_pCmdOp->GetSelectedComponents();

	UpdateData(true);

	// Get owner BrepModel
	HPS::Component ownerComp = view->GetOwnerBrepModel(selCompArr[0]);
	A3DRiBrepModel* pRiBrepModel = HPS::Exchange::Component(ownerComp).GetExchangeEntity();

#ifdef USING_EXCHANGE_PARASOLID
	// Get selected edges
	std::vector<PK_EDGE_t>edges;
	for (int i = 0; i < selCompArr.size(); i++)
	{
		HPS::Component comp = HPS::Component(selCompArr[i]);
		if (ownerComp == view->GetOwnerBrepModel(selCompArr[i]))
			edges.push_back(((HPS::Parasolid::Component)selCompArr[i]).GetParasolidEntity());
	}

	if (m_pProcess->BlendR(m_dBlendR, edges.size(), edges.data(), pRiBrepModel))
	{
		// PsComponent mapper
		HPS::Component psBodyComp = view->GetOwnerPSBodyCompo(selCompArr[0]);
		int body = ((HPS::Parasolid::Component)psBodyComp).GetParasolidEntity();
		view->InitPsBodyMap(body, psBodyComp);
		
		// Tessellate
		HPS::Parasolid::FacetTessellationKit ftk = HPS::Parasolid::FacetTessellationKit::GetDefault();
		HPS::Parasolid::LineTessellationKit ltk = HPS::Parasolid::LineTessellationKit::GetDefault();
		HPS::Parasolid::Component(psBodyComp).Tessellate(ftk, ltk);
	}
#else
	// Get selected edges
	std::vector<A3DTopoEdge*> pTopoEdges;
	for (int i = 0; i < selCompArr.size(); i++)
	{
		if (ownerComp == view->GetOwnerBrepModel(selCompArr[i]))
			pTopoEdges.push_back(HPS::Exchange::Component(selCompArr[i]).GetExchangeEntity());
	}

	if (0 < pTopoEdges.size())
	{
		if (m_pProcess->BlendR(m_dBlendR, pRiBrepModel, pTopoEdges.size(), pTopoEdges.data()))
		{
			// Reload
			HPS::Exchange::ReloadNotifier notifier = HPS::Exchange::Component(ownerComp).Reload();
			notifier.Wait();
		}
	}

#endif
	view->GetCanvas().Update();

	// Show process time
	auto t1 = std::chrono::system_clock::now();

	auto dur1 = t1 - t0;
	auto msec1 = std::chrono::duration_cast<std::chrono::milliseconds>(dur1).count();

	wchar_t wcsbuf[256];
	swprintf(wcsbuf, sizeof(wcsbuf), L"Process time: %d msec", msec1);
	view->ShowMessage(wcsbuf);

	DestroyWindow();
}

void BlendDlg::OnCancel()
{
	m_pCmdOp->Unhighlight();

	DestroyWindow();
}

void BlendDlg::PostNcDestroy()
{
	delete this;
}

void BlendDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT_BLEND_R, m_dBlendR);
	DDX_Control(pDX, IDC_LIST_PS_TAG, m_psTagListBox);
}



BEGIN_MESSAGE_MAP(BlendDlg, CDialogEx)
	ON_WM_TIMER()
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_BUTTON_CLEAR, &BlendDlg::OnBnClickedButtonClear)
END_MESSAGE_MAP()




void BlendDlg::OnTimer(UINT_PTR nIDEvent)
{
	if (m_pCmdOp->m_bUpdated)
	{
		UpdateData(true);

		int nCount = m_psTagListBox.GetCount();
		for (int i = nCount - 1; i > -1; i--)
			m_psTagListBox.DeleteString(i);

		HPS::ComponentArray selCompArr = m_pCmdOp->GetSelectedComponents();

		for (int i = 0; i < selCompArr.size(); i++)
		{
			int iEnt;

#ifdef USING_EXCHANGE_PARASOLID
			iEnt = ((HPS::Parasolid::Component)selCompArr[i]).GetParasolidEntity();
#else
			// Get owner BrepModel
			HPS::Component ownerComp = view->GetOwnerBrepModel(selCompArr[0]);
			A3DRiBrepModel* pRiBrepModel = HPS::Exchange::Component(ownerComp).GetExchangeEntity();

			A3DTopoEdge* pTopoEdge = HPS::Exchange::Component(selCompArr[i]).GetExchangeEntity();
			iEnt = m_pProcess->GetEntityTag(pRiBrepModel, pTopoEdge, false);
#endif
			CString sEnt;
			if (0 < iEnt)
				sEnt.Format(_T("%d"), iEnt);
			else
				sEnt = "UNKNOWN";
			m_psTagListBox.AddString(sEnt);
		}

		CButton* okBtn = (CButton*)GetDlgItem(IDOK);
		if (0 < selCompArr.size())
			okBtn->EnableWindow(TRUE);
		else
			okBtn->EnableWindow(FALSE);

		UpdateData(false);

		m_pCmdOp->m_bUpdated = false;
	}
	CDialogEx::OnTimer(nIDEvent);
}


void BlendDlg::OnClose()
{
	if (m_timerID != 0)
	{
		BOOL err = KillTimer(m_timerID);
		m_timerID = 0;
	}

	CDialogEx::OnClose();
}


void BlendDlg::OnBnClickedButtonClear()
{
	m_pCmdOp->ClearSelection();
	UpdateData(false);
}
