#include "stdafx.h"
#include "ClickEntitiesCmdOp.h"

ClickEntitiesCmdOp::ClickEntitiesCmdOp(HPS::Component::ComponentType entityType, CHPSView* in_view, void* pProcess, 
	bool single, HPS::MouseButtons button, HPS::ModifierKeys modifiers)
	: Operator(button, modifiers)
	, view(in_view)
	, m_compType(entityType)
	, m_bIsSingleSelection(single)
	, m_eClickCompType(ClickCompType::CLICK_BODY)
	, m_bUpdated(false)
{
	if (HPS::Component::ComponentType::ParasolidTopoBody == m_compType ||
		HPS::Component::ComponentType::ExchangeRIBRepModel == m_compType)
	{
		m_selectionLevel = HPS::Selection::Level::Segment;
	}
	else
	{
		m_selectionLevel = HPS::Selection::Level::Subentity;
	}

#ifdef USING_EXCHANGE_PARASOLID
	m_pProcess = (ExPsProcess*)pProcess;
#else
	m_pProcess = (ExProcess*)pProcess;
#endif

	//! [build_highlight_style]
	m_highlight_options = HPS::HighlightOptionsKit::GetDefault();
	m_highlight_options.SetStyleName("highlight_style");
	m_highlight_options.SetSubentityHighlighting(false);
	m_highlight_options.SetOverlay(HPS::Drawing::Overlay::InPlace);
	//! [build_highlight_style]
}

ClickEntitiesCmdOp::~ClickEntitiesCmdOp()
{
}

void ClickEntitiesCmdOp::OnViewDetached(HPS::View const& in_view)
{
	Unhighlight();
}

bool ClickEntitiesCmdOp::OnMouseDown(HPS::MouseState const& in_state)
{
	if (!in_state.GetButtons().Left())
		return false;

	m_clickPnt = in_state.GetLocation();
	m_bIsActive = true;

	return false;
}

bool ClickEntitiesCmdOp::OnMouseUp(HPS::MouseState const& in_state)
{
	if (!m_bIsActive)
		return false;

	HPS::Vector sub = m_clickPnt.operator-(in_state.GetLocation());
	double len = sub.Length();
	if (1.0e-6 < len)
		return false;

	if (m_bIsSingleSelection)
	{
		Unhighlight();
		HPS::ComponentArray().swap(m_selCompArr);
		HPS::ComponentPathArray().swap(m_selCompPathArr);

	}

	HPS::SelectionOptionsKit selection_options;
	selection_options.SetAlgorithm(HPS::Selection::Algorithm::Analytic);
	selection_options.SetLevel(m_selectionLevel).SetSorting(true);

	HPS::SelectionResults selection_results;
	size_t number_of_selected_items = in_state.GetEventSource().GetSelectionControl().SelectByPoint(in_state.GetLocation(), selection_options, selection_results);

	if (0 < number_of_selected_items)
	{

		HPS::SelectionResultsIterator it = selection_results.GetIterator();
		while (it.IsValid())
		{
			HPS::Key selected_key;
			it.GetItem().ShowSelectedItem(selected_key);

			HPS::CADModel cad_model = view->GetDocument()->GetCADModel();
			HPS::Component selComp = cad_model.GetComponentFromKey(selected_key);
			HPS::Component::ComponentType comptype = selComp.GetComponentType();

			HPS::ComponentPath compPath = cad_model.GetComponentPath(it.GetItem());

			if (ClickCompType::CLICK_PART == m_eClickCompType)
			{
				// Get PO of the RiBrepModel from component path 
				if (!compPath.Empty())
				{
					HPS::ComponentArray compArr = compPath.GetComponents();

					do
					{
						HPS::Component poComp = compArr[0];
						HPS::Component::ComponentType compType = poComp.GetComponentType();
						if (HPS::Component::ComponentType::ExchangeProductOccurrence == compType)
						{
							selComp = poComp;
							compPath = HPS::ComponentPath(compArr);
							break;
						}

						compArr.erase(compArr.begin());
					} while (compArr.size());
				}
			}

			if (m_compType == comptype)
			{
				bool bFlg = false;
				for (int i = 0; i < m_selCompArr.size(); i++)
				{
					if (m_selCompArr[i] == selComp)
					{
						m_selCompArr.erase(std::cbegin(m_selCompArr) + i);
						m_selCompPathArr.erase(std::cbegin(m_selCompPathArr) + i);
						bFlg = true;
						break;
					}
				}

				if (!bFlg)
				{
					m_selCompArr.push_back(selComp);
					m_selCompPathArr.push_back(compPath);

					// Make the selected component get highlighted in the model browser
					m_highlight_options.SetNotification(true);
					compPath.Highlight(view->GetCanvas(), m_highlight_options);
				}
				else
				{
					Unhighlight();
					for (int i = 0; i < m_selCompPathArr.size(); i++)
					{
						m_highlight_options.SetNotification(true);
						m_selCompPathArr[i].Highlight(view->GetCanvas(), m_highlight_options);
					}
				}
				GetAttachedView().Update();

				m_bUpdated = true;
				return true;
			}
			it.Next();
		}
	}
	return false;
}

void ClickEntitiesCmdOp::ClearSelection()
{
	HPS::ComponentArray().swap(m_selCompArr);
	HPS::ComponentPathArray().swap(m_selCompPathArr);

	Unhighlight();
	GetAttachedView().Update();
}

void ClickEntitiesCmdOp::Unhighlight()
{
	view->GetCanvas().GetWindowKey().GetHighlightControl().Unhighlight(m_highlight_options);
	view->GetCanvas().Update(HPS::Window::UpdateType::Complete);
}
